@@ -3,6 +3,8 @@ from rlockertools.exceptions import BadRequestError, TimeoutReachedForLockingRes
 from rlockertools.utils import prettify_output
 import requests
 import json
+import time
+
 
 class ResourceLocker:
     def __init__(self, instance_url, token):
@@ -12,16 +14,16 @@ class ResourceLocker:
         self.check_connection()
 
         self.endpoints = {
-            'resources'         : f'{self.instance_url}/api/resources',
-            'retrieve_resource' : f'{self.instance_url}/api/resource/retrieve_entrypoint/',
-            'resource'          : f'{self.instance_url}/api/resource/',
-            'rqueue'            : f'{self.instance_url}/api/rqueue/',
-            'rqueues'           : f'{self.instance_url}/api/rqueues',
+            'resources': f'{self.instance_url}/api/resources',
+            'retrieve_resource': f'{self.instance_url}/api/resource/retrieve_entrypoint/',
+            'resource': f'{self.instance_url}/api/resource/',
+            'rqueue': f'{self.instance_url}/api/rqueue/',
+            'rqueues': f'{self.instance_url}/api/rqueues',
         }
 
         self.headers = {
-          'Content-Type': 'application/json',
-          'Authorization': f'Token {self.token}'
+            'Content-Type': 'application/json',
+            'Authorization': f'Token {self.token}'
         }
 
     def check_connection(self):
@@ -33,26 +35,30 @@ class ResourceLocker:
         '''
         req = requests.get(self.instance_url)
         if req.status_code == 200:
-            print({'CONNECTION' : 'OK'})
+            print({'CONNECTION': 'OK'})
             return
         else:
-            #Raise Connection Error if no 200
+            # Raise Connection Error if no 200
             raise ConnectionError
 
-    def retrieve_and_lock(self, search_string, signoff, priority, timeout=None):
+    def find_resource(self, search_string, signoff, priority, link=None, timeout=None):
         '''
 
         :param search_string:
         :param signoff:
         :param priority:
+        :param link:
         :param timeout:
         :return:
         '''
         final_endpoint = self.endpoints['retrieve_resource'] + search_string
         data = {
-            "priority" : priority,
-            "signoff" : signoff
+            "priority": priority,
+            "signoff": signoff,
         }
+        if link:
+            data['link'] = link
+
         data_json = json.dumps(data)
 
         try:
@@ -77,7 +83,6 @@ class ResourceLocker:
         final_endpoint = self.endpoints['resource'] + lockable_resource['name']
         newjson = json.dumps(lockable_resource)
 
-
         req = requests.put(final_endpoint, headers=self.headers, data=newjson)
         return req
 
@@ -108,7 +113,7 @@ class ResourceLocker:
         '''
         req = requests.get(self.endpoints['resources'], headers=self.headers)
         if req.status_code == 200:
-            #json.loads returns it to a dictionary:
+            # json.loads returns it to a dictionary:
             req_dict = json.loads(req.text.encode('utf8'))
             return req_dict
         else:
@@ -134,21 +139,6 @@ class ResourceLocker:
         '''
         return next(self.filter_lockable_resource(lambda_expression=lambda_expression))
 
-    def get_current_queue(self):
-        '''
-        Get the latest current queue that is created, without filtering its status,
-            just the latest queue ID
-        :return rqueue JSON object
-        '''
-        req = requests.get(self.endpoints['rqueues'], headers=self.headers)
-        if req.status_code == 200:
-            #json.loads returns it to a dictionary
-            req_dict = json.loads(req.text.encode('utf8'))
-            return req_dict
-        else:
-            print(f"There were some errors from the Resource Locker server:")
-            prettify_output(req.text)
-
     def abort_queue(self, queue_id, abort_msg=None):
         '''
         A method to send a POST request to abort the queue that was created, and expected
@@ -161,11 +151,83 @@ class ResourceLocker:
         if req.status_code == 200:
             data_json = json.dumps(
                 {
-                    'status' : 'ABORTED',
-                    'description' : abort_msg,
+                    'status': 'ABORTED',
+                    'description': abort_msg,
                 }
             )
 
             req = requests.put(final_endpoint, headers=self.headers, data=data_json)
             return req
 
+    def change_queue(self, queue_id, status):
+        '''
+        A method to send a POST request to change the status of the queue.
+        :return: req
+        '''
+        final_endpoint = self.endpoints['rqueue'] + str(queue_id)
+        req = requests.get(final_endpoint, headers=self.headers)
+        if req.status_code == 200:
+            data_json = json.dumps(
+                {
+                    'status': status
+                }
+            )
+
+            req = requests.put(final_endpoint, headers=self.headers, data=data_json)
+            return req
+
+        return req
+
+    def get_queues(self, status=None):
+        final_endpoint = self.endpoints['rqueues'] + f'?status={status}' if status else self.endpoints['rqueues']
+        req = requests.get(final_endpoint, headers=self.headers)
+        if req.status_code == 200:
+            # json.loads returns it to a dictionary
+            req_dict = json.loads(req.text.encode('utf8'))
+            return req_dict
+
+    def get_queue(self, queue_id):
+        '''
+        Return queue JSONIFIED by the given queue_id
+        :param queue_id:
+        :return:
+        '''
+        final_endpoint = self.endpoints['rqueue'] + str(queue_id)
+        req = requests.get(final_endpoint, headers=self.headers)
+        if req.status_code == 200:
+            return req.json()
+
+        return None
+
+    def wait_until_status(self, queue_id, status,
+                          interval=15, attempts=120, silent=True):
+        '''
+        A method that uses multiple retries until a status of queue is achieved
+        :param queue_id:
+        :param status:
+        :param interval:
+        :param attempts:
+        :param silent: If timeout is reached (attempts * interval), then
+            it will silently return None rather than raising Exception.
+        :return:
+        '''
+        for attempt in range(attempts):
+            queue_to_check = self.get_queue(queue_id)
+            if not queue_to_check:
+                raise Exception(f'Queue {queue_id} does not exist on the server!')
+
+            if queue_to_check.get('status') == status:
+                return True
+
+            else:
+                print(f"{queue_id} is not in status {status} yet"
+                      f"More info about the queue: \n"
+                      f"{self.instance_url}\pendingrequests")
+
+                time.sleep(interval)
+        else:
+            if silent:
+                return None
+            else:
+                raise Exception("Timeout Reached! \n"
+                                f"Status of the queue is not {status}!")
